# unpack.py
# Author: Meghan Clark

from .msgtypes import *
import binascii


# Creates a LIFX Message out of packed binary data
# If the message type is not one of the officially released ones above, it will create just a Message out of it
# If it's not in the LIFX protocol format, uhhhhh...we'll put that on a to-do list.
def unpack_lifx_message(packed_message):
    header_str = packed_message[0:HEADER_SIZE_BYTES]
    payload_str = packed_message[HEADER_SIZE_BYTES:]

    size = struct.unpack("H", header_str[0:2])[0]
    flags = struct.unpack("H", header_str[2:4])[0]
    origin = (flags >> 14) & 3
    tagged = (flags >> 13) & 1
    addressable = (flags >> 12) & 1
    protocol = flags & 4095
    source_id = struct.unpack("I", header_str[4:8])[0]
    target_addr = ":".join(
        [("%02x" % b) for b in struct.unpack("B" * 6, header_str[8:14])]
    )
    response_flags = struct.unpack("B", header_str[22:23])[0]
    ack_requested = response_flags & 2
    response_requested = response_flags & 1
    seq_num = struct.unpack("B", header_str[23:24])[0]
    message_type = struct.unpack("H", header_str[32:34])[0]

    message = None
    if message_type == MSG_IDS[GetService]:
        message = GetService(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateService]:
        service = struct.unpack("B", payload_str[0:1])[0]
        port = struct.unpack("I", payload_str[1:5])[0]
        payload = {"service": service, "port": port}
        message = StateService(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetHostInfo]:
        message = GetHostInfo(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateHostInfo]:
        signal = struct.unpack("f", payload_str[0:4])[0]
        tx = struct.unpack("I", payload_str[4:8])[0]
        rx = struct.unpack("I", payload_str[8:12])[0]
        reserved1 = struct.unpack("h", payload_str[12:14])[0]
        payload = {"signal": signal, "tx": tx, "rx": rx, "reserved1": reserved1}
        message = StateHostInfo(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetHostFirmware]:
        message = GetHostFirmware(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateHostFirmware]:
        build = struct.unpack("Q", payload_str[0:8])[0]
        reserved1 = struct.unpack("Q", payload_str[8:16])[0]
        version = struct.unpack("I", payload_str[16:20])[0]
        payload = {"build": build, "reserved1": reserved1, "version": version}
        message = StateHostFirmware(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetWifiInfo]:
        message = GetWifiInfo(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateWifiInfo]:
        signal = struct.unpack("f", payload_str[0:4])[0]
        tx = struct.unpack("I", payload_str[4:8])[0]
        rx = struct.unpack("I", payload_str[8:12])[0]
        reserved1 = struct.unpack("h", payload_str[12:14])[0]
        payload = {"signal": signal, "tx": tx, "rx": rx, "reserved1": reserved1}
        message = StateWifiInfo(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetWifiFirmware]:
        message = GetWifiFirmware(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateWifiFirmware]:
        build = struct.unpack("Q", payload_str[0:8])[0]
        reserved1 = struct.unpack("Q", payload_str[8:16])[0]
        version = struct.unpack("I", payload_str[16:20])[0]
        payload = {"build": build, "reserved1": reserved1, "version": version}
        message = StateWifiFirmware(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetPower]:
        message = GetPower(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[SetPower]:
        power_level = struct.unpack("H", payload_str[0:2])[0]
        payload = {"power_level": power_level}
        message = SetPower(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StatePower]:
        power_level = struct.unpack("H", payload_str[0:2])[0]
        payload = {"power_level": power_level}
        message = StatePower(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetLabel]:
        message = GetLabel(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[SetLabel]:
        label = binascii.unhexlify(
            "".join(
                [
                    "%2.2x" % (b & 0x000000FF)
                    for b in struct.unpack("b" * 32, payload_str[0:32])
                ]
            )
        )
        payload = {"label": label}
        message = SetLabel(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateLabel]:
        label = binascii.unhexlify(
            "".join(
                [
                    "%2.2x" % (b & 0x000000FF)
                    for b in struct.unpack("b" * 32, payload_str[0:32])
                ]
            )
        )
        payload = {"label": label}
        message = StateLabel(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetLocation]:
        message = GetLocation(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateLocation]:
        location = [b for b in struct.unpack("B" * 16, payload_str[0:16])]
        label = binascii.unhexlify(
            "".join(
                [
                    "%2.2x" % (b & 0x000000FF)
                    for b in struct.unpack("b" * 32, payload_str[16:48])
                ]
            )
        )
        updated_at = struct.unpack("Q", payload_str[48:56])[0]
        payload = {"location": location, "label": label, "updated_at": updated_at}
        message = StateLocation(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetGroup]:
        message = GetGroup(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateGroup]:
        group = [b for b in struct.unpack("B" * 16, payload_str[0:16])]
        label = binascii.unhexlify(
            "".join(
                [
                    "%2.2x" % (b & 0x000000FF)
                    for b in struct.unpack("b" * 32, payload_str[16:48])
                ]
            )
        )
        updated_at = struct.unpack("Q", payload_str[48:56])[0]
        payload = {"group": group, "label": label, "updated_at": updated_at}
        message = StateGroup(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetVersion]:
        message = GetVersion(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateVersion]:
        vendor = struct.unpack("I", payload_str[0:4])[0]
        product = struct.unpack("I", payload_str[4:8])[0]
        version = struct.unpack("I", payload_str[8:12])[0]
        payload = {"vendor": vendor, "product": product, "version": version}
        message = StateVersion(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetInfo]:
        message = GetInfo(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateInfo]:
        time = struct.unpack("Q", payload_str[0:8])[0]
        uptime = struct.unpack("Q", payload_str[8:16])[0]
        downtime = struct.unpack("Q", payload_str[16:24])[0]
        payload = {"time": time, "uptime": uptime, "downtime": downtime}
        message = StateInfo(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[Acknowledgement]:
        message = Acknowledgement(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[EchoRequest]:
        byte_array_len = len(payload_str)
        byte_array = [
            b
            for b in struct.unpack("B" * byte_array_len, payload_str[0:byte_array_len])
        ]
        payload = {"byte_array": byte_array}
        message = EchoRequest(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[EchoResponse]:
        byte_array_len = len(payload_str)
        byte_array = [
            b
            for b in struct.unpack("B" * byte_array_len, payload_str[0:byte_array_len])
        ]
        payload = {"byte_array": byte_array}
        message = EchoResponse(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightGet]:
        message = LightGet(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightSetColor]:
        color = struct.unpack("H" * 4, payload_str[0:8])
        duration = struct.unpack("I", payload_str[8:12])[0]
        payload = {"color": color, "duration": duration}
        message = LightSetColor(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightState]:
        color = struct.unpack("H" * 4, payload_str[0:8])
        reserved1 = struct.unpack("H", payload_str[8:10])[0]
        power_level = struct.unpack("H", payload_str[10:12])[0]
        label = binascii.unhexlify(
            "".join(
                [
                    "%2.2x" % (b & 0x000000FF)
                    for b in struct.unpack("b" * 32, payload_str[12:44])
                ]
            )
        )
        reserved2 = struct.unpack("Q", payload_str[44:52])[0]
        payload = {
            "color": color,
            "reserved1": reserved1,
            "power_level": power_level,
            "label": label,
            "reserved2": reserved2,
        }
        message = LightState(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightGetPower]:
        message = LightGetPower(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightSetPower]:
        power_level = struct.unpack("H", payload_str[0:2])[0]
        duration = struct.unpack("I", payload_str[2:6])[0]
        payload = {"power_level": power_level, "duration": duration}
        message = LightSetPower(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightStatePower]:
        power_level = struct.unpack("H", payload_str[0:2])[0]
        payload = {"power_level": power_level}
        message = LightStatePower(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightGetInfrared]:  # 120
        message = LightGetInfrared(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightStateInfrared]:  # 121
        infrared_brightness = struct.unpack("H", payload_str[0:2])[0]
        payload = {"infrared_brightness": infrared_brightness}
        message = LightStateInfrared(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[LightSetInfrared]:  # 122
        infrared_brightness = struct.unpack("H", payload_str[0:2])[0]
        payload = {"infrared_brightness": infrared_brightness}
        message = LightSetInfrared(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetHevCycle]:  # 142
        message = GetHevCycle(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[SetHevCycle]:  # 143
        enable, duration = struct.unpack("<BI", payload_str[:5])
        payload = {"enable": enable == 1, "duration": duration}
        message = SetHevCycle(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateHevCycle]:  # 144
        duration, remaining, last_power = struct.unpack("<IIB", payload_str[:9])
        payload = {
            "duration": duration,
            "remaining": remaining,
            "last_power": last_power == 1,
        }
        message = StateHevCycle(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetHevCycleConfiguration]:  # 145
        message = GetHevCycleConfiguration(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[SetHevCycleConfiguration]:  # 146
        indication, duration = struct.unpack("<BI", payload_str[:5])
        payload = {"indication": indication == 1, "duration": duration}
        message = SetHevCycleConfiguration(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateHevCycleConfiguration]:  # 147
        indication, duration = struct.unpack("<BI", payload_str[:5])
        payload = {"indication": indication == 1, "duration": duration}
        message = StateHevCycleConfiguration(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[GetLastHevCycleResult]:  # 148
        message = GetLastHevCycleResult(
            target_addr, source_id, seq_num, {}, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateLastHevCycleResult]:  # 149
        (result,) = struct.unpack("<B", payload_str[:1])
        payload = {"result": result}
        message = StateLastHevCycleResult(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneStateZone]:  # 503
        count = struct.unpack("c", payload_str[0:1])[0]
        count = ord(count)  # 8 bit
        index = struct.unpack("c", payload_str[1:2])[0]
        index = ord(index)  # 8 bit
        color = struct.unpack("H" * 4, payload_str[2:10])
        payload = {"count": count, "index": index, "color": color}
        message = MultiZoneStateZone(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneStateMultiZone]:  # 506
        count = struct.unpack("c", payload_str[0:1])[0]
        count = ord(count)  # 8 bit
        index = struct.unpack("c", payload_str[1:2])[0]
        index = ord(index)  # 8 bit
        colors = []
        for i in range(8):
            color = struct.unpack("H" * 4, payload_str[2 + (i * 8) : 10 + (i * 8)])
            colors.append(color)
        payload = {"count": count, "index": index, "color": colors}
        message = MultiZoneStateMultiZone(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneGetMultiZoneEffect]:  # 507
        _, effect, _, speed, duration, _, _, _, direction, _, _ = struct.unpack(
            "<IBHIQIIBBBB", payload_str[:31]
        )
        payload = {
            "effect": effect,
            "speed": speed,
            "duration": duration,
            "direction": direction,
        }
        message = MultiZoneGetMultiZoneEffect(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneSetMultiZoneEffect]:  # 508
        _, effect, _, speed, duration, _, _, _, direction, _, _ = struct.unpack(
            "<IBHIQIIBBBB", payload_str[:31]
        )
        payload = {
            "effect": effect,
            "speed": speed,
            "duration": duration,
            "direction": direction,
        }
        message = MultiZoneSetMultiZoneEffect(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneStateMultiZoneEffect]:  # 509
        (
            instanceid,
            effect,
            _,
            speed,
            duration,
            _,
            _,
        ) = struct.unpack("<IBHIQII", payload_str[:27])
        direction = struct.unpack("I", payload_str[31:35])[0]
        payload = {
            "instanceid": instanceid,
            "effect": effect,
            "speed": speed,
            "duration": duration,
            "direction": direction,
        }
        message = MultiZoneStateMultiZoneEffect(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[MultiZoneStateExtendedColorZones]:  # 512
        zones_count = struct.unpack("H", payload_str[0:2])[0]
        zone_index = struct.unpack("H", payload_str[2:4])[0]
        colors_count = struct.unpack("B", payload_str[4:5])[0]
        colors = []
        for i in range(82):
            color = struct.unpack("H" * 4, payload_str[5 + (i * 8) : 13 + (i * 8)])
            colors.append(color)

        payload = {
            "zones_count": zones_count,
            "zone_index": zone_index,
            "colors_count": colors_count,
            "colors": colors,
        }
        message = MultiZoneStateExtendedColorZones(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileGetDeviceChain]:  # 701
        message = TileGetDeviceChain(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileStateDeviceChain]:  # 702
        start_index = struct.unpack("B", payload_str[0:1])[0]
        tile_devices_count = struct.unpack(
            "B", payload_str[len(payload_str) - 1 : len(payload_str)]
        )[0]

        tile_devices = []
        for i in range(tile_devices_count):
            tile_devices.append(getTile(payload_str[1 + (i * 55) : 55 + (i * 55)]))

        payload = {
            "start_index": start_index,
            "tile_devices": tile_devices,
            "tile_devices_count": tile_devices_count,
        }
        message = TileStateDeviceChain(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileGet64]:  # 707
        tile_index = struct.unpack("B", payload_str[0:1])[0]
        length = struct.unpack("B", payload_str[1:2])[0]
        x = struct.unpack("B", payload_str[3:4])[0]
        y = struct.unpack("B", payload_str[4:5])[0]
        width = struct.unpack("B", payload_str[5:6])[0]
        payload = {
            "tile_index": tile_index,
            "length": length,
            "x": x,
            "y": y,
            "width": width,
        }
        message = TileGet64(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileState64]:  # 711
        tile_index = struct.unpack("B", payload_str[0:1])[0]
        x = struct.unpack("B", payload_str[2:3])[0]
        y = struct.unpack("B", payload_str[3:4])[0]
        width = struct.unpack("B", payload_str[4:5])[0]
        colors = []
        for i in range(64):
            color = struct.unpack("H" * 4, payload_str[5 + (i * 8) : 13 + (i * 8)])
            colors.append(color)

        payload = {
            "tile_index": tile_index,
            "x": x,
            "y": y,
            "width": width,
            "colors": colors,
        }
        message = TileState64(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileSet64]:  # 715
        tile_index = struct.unpack("B", payload_str[0:1])[0]
        length = struct.unpack("B", payload_str[1:2])[0]
        x = struct.unpack("B", payload_str[3:4])[0]
        y = struct.unpack("B", payload_str[4:5])[0]
        width = struct.unpack("B", payload_str[5:6])[0]
        duration = struct.unpack("I", payload_str[6:10])[0]
        colors = []
        for i in range(64):
            color = struct.unpack("H" * 4, payload_str[10 + (i * 8) : 18 + (i * 8)])
            colors.append(color)
        payload = {
            "tile_index": tile_index,
            "length": length,
            "x": x,
            "y": y,
            "width": width,
            "duration": duration,
            "colors": colors,
        }
        message = TileSet64(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[TileStateTileEffect]:  # 720
        instanceid = struct.unpack("I", payload_str[1:5])[0]
        effect = struct.unpack("B", payload_str[5:6])[0]
        speed = struct.unpack("I", payload_str[6:10])[0]
        duration = struct.unpack("Q", payload_str[10:18])[0]
        sky_type = struct.unpack("B", payload_str[26:27])[0]
        cloud_saturation_min = struct.unpack("B", payload_str[30:31])[0]
        cloud_saturation_max = struct.unpack("B", payload_str[34:35])[0]
        palette_count = struct.unpack("B", payload_str[58:59])[0]
        palette = []
        for i in range(16):
            color = struct.unpack("H" * 4, payload_str[59 + (i * 8) : 67 + (i * 8)])
            palette.append(color)

        payload = {
            "instanceid": instanceid,
            "effect": effect,
            "speed": speed,
            "duration": duration,
            "sky_type": sky_type,
            "cloud_saturation_min": cloud_saturation_min,
            "cloud_saturation_max": cloud_saturation_max,
            "palette_count": palette_count,
            "palette": palette,
        }
        message = TileStateTileEffect(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateRPower]:  # 818
        relay_index = struct.unpack("B", payload_str[:1])[0]
        level = struct.unpack(">H", payload_str[1:])[0]
        payload = {"relay_index": relay_index, "level": level}
        message = StateRPower(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateButton]:  # 907
        count = struct.unpack("B", payload_str[:1])[0]
        index = struct.unpack("B", payload_str[1:2])[0]
        buttons_count = struct.unpack("B", payload_str[2:3])[0]

        # always an array of 8 buttons
        buttons = []
        for i in range(8):
            # each button is 101 bytes
            button_bytes = payload_str[3 + (i * 101) : 104 + (i * 101)]
            actions_count = struct.unpack("B", button_bytes[:1])[0]
            # each button has 5 actions, size 100 bytes each
            button_actions = []
            for j in range(5):
                button_action_bytes = button_bytes[1 + (j * 20) : 21 + (j * 20)]

                button_gesture = struct.unpack("H", button_action_bytes[:2])[0]
                button_gesture_enum = ButtonGesture(button_gesture)

                button_target_type = struct.unpack("H", button_action_bytes[2:4])[0]
                button_target_type_enum = ButtonTargetType(button_target_type)

                button_target = button_action_bytes[4:]
                button_target_properties = {
                    "type": button_target_type_enum,
                }
                if button_target_type_enum == ButtonTargetType.RELAYS:
                    button_target_properties["relays_count"] = struct.unpack(
                        "B", button_target[:1]
                    )[0]
                    button_target_properties["relays"] = struct.unpack(
                        "B" * 15, button_target[1:]
                    )
                elif button_target_type_enum == ButtonTargetType.DEVICE:
                    button_target_properties["serial"] = struct.unpack(
                        "B" * 6, button_target[:6]
                    )
                    button_target_properties["reserved"] = struct.unpack(
                        "B" * 10, button_target[6:]
                    )
                elif button_target_type_enum == ButtonTargetType.LOCATION:
                    button_target_properties["location_id"] = struct.unpack(
                        "B" * 16, button_target[:16]
                    )
                elif button_target_type_enum == ButtonTargetType.GROUP:
                    button_target_properties["group_id"] = struct.unpack(
                        "B" * 16, button_target[:16]
                    )
                elif button_target_type_enum == ButtonTargetType.SCENE:
                    button_target_properties["scene_id"] = struct.unpack(
                        "B" * 16, button_target[:16]
                    )
                elif button_target_type_enum == ButtonTargetType.DEVICE_RELAYS:
                    button_target_properties["serial"] = struct.unpack(
                        "B" * 6, button_target[:6]
                    )
                    button_target_properties["relays_count"] = struct.unpack(
                        "B", button_target[6:7]
                    )[0]
                    button_target_properties["relays"] = struct.unpack(
                        "B" * 9, button_target[7:]
                    )
                button_action = {
                    "button_gesture": button_gesture_enum,
                    "button_target_type": button_target_type_enum,
                    "button_target": button_target_properties,
                }
                button_actions.append(button_action)
            button = {
                "actions_count": actions_count,
                "button_actions": button_actions,
            }
            buttons.append(button)

        payload = {
            "count": count,
            "index": index,
            "buttons_count": buttons_count,
            "buttons": buttons,
        }
        message = StateButton(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )

    elif message_type == MSG_IDS[StateButtonConfig]:  # 911
        haptic_duration_ms = struct.unpack("B", payload_str[:1])[0]

        backlight_on_color_values = payload_str[1:9]
        backlight_on_color = getBacklightColor(backlight_on_color_values)

        backlight_off_color_values = payload_str[9:17]
        backlight_off_color = getBacklightColor(backlight_off_color_values)

        payload = {
            "haptic_duration_ms": haptic_duration_ms,
            "backlight_on_color": backlight_on_color,
            "backlight_off_color": backlight_off_color,
        }

        message = StateButtonConfig(
            target_addr, source_id, seq_num, payload, ack_requested, response_requested
        )
    else:
        message = Message(
            message_type,
            target_addr,
            source_id,
            seq_num,
            ack_requested,
            response_requested,
        )

    message.size = size
    message.origin = origin
    message.tagged = tagged
    message.addressable = addressable
    message.protocol = protocol
    message.source_id = source_id
    message.header = header_str
    message.payload = payload_str
    message.packed_message = packed_message

    return message


class ButtonGesture(Enum):
    NONE = 0
    PRESS = 1
    HOLD = 2
    PRESS_PRESS = 3
    PRESS_HOLD = 4
    HOLD_HOLD = 5


class ButtonTargetType(Enum):
    RESERVED = 0
    RESERVED_1 = 1
    RELAYS = 2
    DEVICE = 3
    LOCATION = 4
    GROUP = 5
    SCENE = 6
    DEVICE_RELAYS = 7


def getTile(payload_str):
    accel_meas_x = struct.unpack("h", payload_str[0:2])[0]
    accel_meas_y = struct.unpack("h", payload_str[2:4])[0]
    accel_meas_z = struct.unpack("h", payload_str[4:6])[0]
    # 6:8
    user_x = struct.unpack("f", payload_str[8:12])[0]
    user_y = struct.unpack("f", payload_str[12:16])[0]
    width = struct.unpack("B", payload_str[16:17])[0]
    height = struct.unpack("B", payload_str[17:18])[0]
    # 18:19
    device_version_vendor = struct.unpack("I", payload_str[19:23])[0]
    device_version_product = struct.unpack("I", payload_str[23:27])[0]
    # 27:31
    firmware_build = struct.unpack("Q", payload_str[31:39])[0]
    # 39:47
    firmware_version_minor = struct.unpack("H", payload_str[47:49])[0]
    firmware_version_major = struct.unpack("H", payload_str[49:51])[0]
    # 51:55

    return {
        "accel_meas_x": accel_meas_x,
        "accel_meas_y": accel_meas_y,
        "accel_meas_z": accel_meas_z,
        "user_x": user_x,
        "user_y": user_y,
        "width": width,
        "height": height,
        "device_version_vendor": device_version_vendor,
        "device_version_product": device_version_product,
        "firmware_build": firmware_build,
        "firmware_version_minor": firmware_version_minor,
        "firmware_version_major": firmware_version_major,
    }


def getBacklightColor(payload_str):
    hue = struct.unpack("H", payload_str[:2])[0]
    saturation = struct.unpack("H", payload_str[2:4])[0]
    brightness = struct.unpack("H", payload_str[4:6])[0]
    kelvin = struct.unpack("H", payload_str[6:8])[0]

    return {
        "hue": hue,
        "saturation": saturation,
        "brightness": brightness,
        "kelvin": kelvin,
    }



from typing import List
from .products_defs import product_map, features_map


class Product:
    def __init__(
        self,
        id: int,
        name: str,
        buttons: bool,
        chain: bool,
        color: bool,
        extended_multizone: bool,
        hev: bool,
        infrared: bool,
        matrix: bool,
        multizone: bool,
        relays: bool,
        max_kelvin: int,
        min_kelvin: int,
        min_ext_mz_firmware: int,
        min_ext_mz_firmware_components: List[int],
        temperature_range: None,
    ):
        self.id = id
        self.name = name
        self.buttons = buttons
        self.chain = chain
        self.color = color
        self.extended_multizone = extended_multizone
        self.hev = hev
        self.infrared = infrared
        self.matrix = matrix
        self.multizone = multizone
        self.relays = relays
        self.max_kelvin = max_kelvin
        self.min_kelvin = min_kelvin
        self.min_ext_mz_firmware = min_ext_mz_firmware
        self.min_ext_mz_firmware_components = min_ext_mz_firmware_components
        self.temperature_range = temperature_range

    def __str__(self):
        return (
            f"Product(id={self.id}, "
            f"name='{self.name}', "
            f"buttons={self.buttons}, "
            f"chain={self.chain}, "
            f"color={self.color}, "
            f"extended_multizone={self.extended_multizone}, "
            f"hev={self.hev}, "
            f"infrared={self.infrared}, "
            f"matrix={self.matrix}, "
            f"multizone={self.multizone}, "
            f"relays={self.relays}, "
            f"max_kelvin={self.max_kelvin}, "
            f"min_kelvin={self.min_kelvin}, "
            f"min_ext_mz_firmware={self.min_ext_mz_firmware}, "
            f"min_ext_mz_firmware_components={self.min_ext_mz_firmware_components}, "
            f"temperature_range={self.temperature_range})"
        )


def create_product_dict(product_map, features_map):
    products_dict = {}
    for product_id, product_name in product_map.items():
        features = features_map[product_id]
        products_dict[product_id] = Product(
            id=product_id,
            name=product_name,
            buttons=features.get("buttons"),
            chain=features.get("chain"),
            color=features.get("color"),
            extended_multizone=features.get("extended_multizone"),
            hev=features.get("hev"),
            infrared=features.get("infrared"),
            matrix=features.get("matrix"),
            multizone=features.get("multizone"),
            relays=features.get("relays"),
            max_kelvin=features.get("max_kelvin"),
            min_kelvin=features.get("min_kelvin"),
            min_ext_mz_firmware=features.get("min_ext_mz_firmware"),
            min_ext_mz_firmware_components=features.get(
                "min_ext_mz_firmware_components"
            ),
            temperature_range=features.get("temperature_range"),
        )
    return products_dict


products_dict = create_product_dict(product_map, features_map)



#!/usr/bin/env python3

import requests
import json
import pprint


product_map = {}
features_map = {}
for url in ["https://raw.githubusercontent.com/LIFX/products/master/products.json"]:
    resp = requests.get(url)
    lifxp = json.loads(resp.text)
    defaults = lifxp[0]["defaults"]

    for product in lifxp[0]["products"]:
        product_id = product["pid"]

        product_map[product_id] = product["name"]

        features = {}
        for feature in defaults:
            if feature in product["features"]:
                features[feature] = product["features"][feature]
            else:
                features[feature] = defaults[feature]

        for upgrade in product.get("upgrades", []):
            features.update(upgrade["features"])

        if features["temperature_range"] is not None:
            features["min_kelvin"] = features["temperature_range"][0]
            features["max_kelvin"] = features["temperature_range"][1]
            del features["temperature_range"]

        if "min_ext_mz_firmware" in product["features"]:
            features["min_ext_mz_firmware"] = product["features"]["min_ext_mz_firmware"]

        if "min_ext_mz_firmware_components" in product["features"]:
            features["min_ext_mz_firmware_components"] = product["features"][
                "min_ext_mz_firmware_components"
            ]

        features_map[product_id] = features

    output = "product_map = {products}\n\nfeatures_map = {features}\n".format(
        products=pprint.pformat(product_map, indent=4),
        features=pprint.pformat(features_map, indent=4),
    )

print(output)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
#
# This application is an example on how to use aiolifx
#
# Copyright (c) 2016 François Wautier
# Copyright (c) 2022 Michael Farrell <micolous+git@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
# of the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies
# or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
# IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
from enum import Enum
import asyncio as aio

import click
import aiolifx as alix
from functools import partial
from time import sleep
from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from InquirerPy.validator import EmptyInputValidator

UDP_BROADCAST_PORT = 56700


# Simple bulb control frpm console
class bulbs:
    """A simple class with a register and unregister methods"""

    def __init__(self):
        self.bulbs = []
        self.boi = None  # bulb of interest

    def register(self, bulb):
        global opts
        bulb.get_label()
        bulb.get_location()
        bulb.get_version()
        bulb.get_group()
        bulb.get_wififirmware()
        bulb.get_hostfirmware()
        self.bulbs.append(bulb)
        self.bulbs.sort(key=lambda x: x.label or x.mac_addr)
        if opts["extra"]:
            bulb.register_callback(lambda y: print("Unexpected message: %s" % str(y)))

    def unregister(self, bulb):
        idx = 0
        for x in list([y.mac_addr for y in self.bulbs]):
            if x == bulb.mac_addr:
                del self.bulbs[idx]
                break
            idx += 1


class DeviceFeatures(Enum):
    INFO = "Info"
    FIRMWARE = "Firmware"
    WIFI = "Wifi"
    UPTIME = "Uptime"
    POWER = "Power"
    WHITE = "White"
    COLOR = "Color"
    PULSE = "Pulse"
    HEV_CYCLE = "HEV Cycle"
    HEV_CONFIGURATION = "HEV Configuration"
    MULTIZONE_FIRMWARE_EFFECT = "Get Multizone Firmware Effect"
    MULTIZONE_FIRMWARE_EFFECT_START_STOP = "Start/Stop Firmware Effect"
    MATRIX_FIRMWARE_EFFECT = "Get Matrix Firmware Effect"
    MATRIX_FIRMWARE_EFFECT_START_STOP = "Start/Stop Firmware Effect"
    RELAYS = "Relays"
    BUTTONS = "Buttons"
    BUTTON_CONFIG = "Button Config"
    REBOOT = "Reboot"


def get_features(device):
    base_options = [
        DeviceFeatures.INFO,
        DeviceFeatures.FIRMWARE,
        DeviceFeatures.WIFI,
        DeviceFeatures.UPTIME,
        DeviceFeatures.REBOOT,
    ]
    features = []
    if alix.aiolifx.products_dict[device].max_kelvin != None:
        features.extend([DeviceFeatures.POWER, DeviceFeatures.WHITE])
    if alix.aiolifx.products_dict[device].color is True:
        features.extend([DeviceFeatures.COLOR, DeviceFeatures.PULSE])
    if alix.aiolifx.products_dict[device].hev is True:
        features.extend([DeviceFeatures.HEV_CYCLE, DeviceFeatures.HEV_CONFIGURATION])
    if alix.aiolifx.products_dict[device].multizone is True:
        features.extend(
            [
                DeviceFeatures.MULTIZONE_FIRMWARE_EFFECT,
                DeviceFeatures.MULTIZONE_FIRMWARE_EFFECT_START_STOP,
            ]
        )
    if alix.aiolifx.products_dict[device].matrix is True:
        features.extend(
            [
                DeviceFeatures.MATRIX_FIRMWARE_EFFECT,
                DeviceFeatures.MATRIX_FIRMWARE_EFFECT_START_STOP,
            ]
        )
    if alix.aiolifx.products_dict[device].relays is True:
        features.append(DeviceFeatures.RELAYS)
    if alix.aiolifx.products_dict[device].buttons is True:
        features.extend([DeviceFeatures.BUTTONS, DeviceFeatures.BUTTON_CONFIG])
    features.extend(base_options)
    return features


async def get_device(devices):
    device_choices = [
        *[Choice(device.mac_addr, name=device.label) for device in devices],
        Choice("back", name="❌ Quit"),
    ]
    device_mac_addr = await inquirer.fuzzy(
        message="Select a device", choices=device_choices
    ).execute_async()
    device = next(
        (device for device in devices if device.mac_addr == device_mac_addr), None
    )
    return device


async def get_feature(device):
    features = get_features(device.product)
    features_choices = [
        *[Choice(feature, name=feature.value) for feature in features],
        (Choice("back", name="❌ Go back to device selection")),
    ]
    option = await inquirer.fuzzy(
        message="Select an option", choices=features_choices
    ).execute_async()
    if option == "back":
        return None
    return option


async def readin():
    while True:
        device = await get_device(MyBulbs.bulbs)
        if device is None:
            break
        feature = await get_feature(device)
        if feature is None:  # if going back
            continue
        if feature == DeviceFeatures.POWER:
            power_level = await inquirer.select(
                message="Select a power level", choices=["On", "Off"]
            ).execute_async()
            device.set_power(power_level.lower())
        elif feature == DeviceFeatures.WHITE:
            brightness = await inquirer.number(
                "Brightness (0 - 100)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=100,
                validate=EmptyInputValidator(),
            ).execute_async()
            while True:
                kelvin = await inquirer.number(
                    "Kelvin (1500 - 9000)",
                    min_allowed=0,
                    max_allowed=9000,
                    validate=EmptyInputValidator(),
                ).execute_async()
                if int(kelvin) < 1500:
                    print("Kelvin must be greater than 1500")
                    continue
                break
            device.set_color(
                [
                    58275,
                    0,
                    int(round((float(brightness) * 65365.0) / 100.0)),
                    float(kelvin),
                ]
            )
        elif feature == DeviceFeatures.COLOR:
            hue = await inquirer.number(
                "Hue (0 - 360)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=360,
                validate=EmptyInputValidator(),
            ).execute_async()
            saturation = await inquirer.number(
                "Saturation (0 - 100)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=100,
                validate=EmptyInputValidator(),
            ).execute_async()
            brightness = await inquirer.number(
                "Brightness (0 - 100)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=100,
                validate=EmptyInputValidator(),
            ).execute_async()
            device.set_color(
                [
                    int(round((float(hue) * 65535.0) / 360.0)),
                    int(round((float(saturation) * 65535.0) / 100.0)),
                    int(round((float(brightness) * 65535.0) / 100.0)),
                    3500,
                ]
            )
            device = None

        elif feature == DeviceFeatures.INFO:
            print(device.device_characteristics_str("    "))
            print(device.device_product_str("    "))
            device = None
        elif feature == DeviceFeatures.FIRMWARE:
            print(device.device_firmware_str("   "))
            device = None
        elif feature == DeviceFeatures.WIFI:
            mypartial = partial(device.device_radio_str)
            device.get_wifiinfo(callb=lambda x, y: print("\n" + mypartial(y)))
            device = None
        elif feature == DeviceFeatures.UPTIME:
            mypartial = partial(device.device_time_str)
            device.get_hostinfo(callb=lambda x, y: print("\n" + mypartial(y)))
            device = None
        elif feature == DeviceFeatures.PULSE:
            hue = await inquirer.number(
                "Hue (0 - 360)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=360,
                validate=EmptyInputValidator(),
            ).execute_async()
            saturation = await inquirer.number(
                "Saturation (0 - 100)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=100,
                validate=EmptyInputValidator(),
            ).execute_async()
            brightness = await inquirer.number(
                "Brightness (0 - 100)",
                replace_mode=True,
                min_allowed=0,
                max_allowed=100,
                validate=EmptyInputValidator(),
            ).execute_async()
            device.set_waveform(
                {
                    "color": [
                        int(round((float(hue) * 65535.0) / 360.0)),
                        int(round((float(saturation) * 65535.0) / 100.0)),
                        int(round((float(brightness) * 65535.0) / 100.0)),
                        3500,
                    ],
                    "transient": 1,
                    "period": 100,
                    "cycles": 30,
                    "skew_ratio": 0,
                    "waveform": 3,
                }
            )
            device = None
        elif feature == DeviceFeatures.HEV_CYCLE:
            device.get_hev_cycle(
                callb=lambda _, r: print(
                    f"\nHEV: duration={r.duration}, "
                    f"remaining={r.remaining}, "
                    f"last_power={r.last_power}"
                )
            )
            device.get_last_hev_cycle_result(
                callb=lambda _, r: print(f"\nHEV result: {r.result_str}")
            )
            set_duration = await inquirer.confirm(
                message="Set duration?", default=False
            ).execute_async()
            if set_duration:
                duration = await inquirer.number(
                    "Duration (seconds)",
                    replace_mode=True,
                    min_allowed=0,
                    validate=EmptyInputValidator(),
                ).execute_async()
                enable = duration >= 0
                if enable:
                    print(f"Running HEV cycle for {duration} second(s)")
                else:
                    print(f"Aborting HEV cycle")
                    duration = 0
                device.set_hev_cycle(
                    enable=enable,
                    duration=duration,
                    callb=lambda _, r: print(
                        f"\nHEV: duration={r.duration}, "
                        f"remaining={r.remaining}, "
                        f"last_power={r.last_power}"
                    ),
                )

            device = None
        elif feature == DeviceFeatures.MATRIX_FIRMWARE_EFFECT:

            print("Getting current firmware effect state from matrix device")
            device.get_tile_effect(
                callb=lambda _, r: print(
                    f"\nCurrent effect={r.effect_str}"
                    f"\nSpeed={r.speed/1000 if getattr(r, 'speed', None) is not None else 0}"
                    f"\nDuration={r.duration/1000000000 if getattr(r, 'duration', None) is not None else 0:4f}"
                    f"\nSky type={r.sky_type_str}"
                    f"\nCloud saturation min={r.cloud_saturation_min}"
                    f"\nCloud saturation max={r.cloud_saturation_max}"
                )
            )
            device = None
        elif feature == DeviceFeatures.MATRIX_FIRMWARE_EFFECT_START_STOP:

            effect = await inquirer.fuzzy(
                message="Effect",
                choices=["Off", "Morph", "Flame", "Sky"],
            ).execute_async()

            if effect.lower() == "sky":
                sky_type = await inquirer.fuzzy(
                    message="Sky Type", choices=["Sunrise", "Sunset", "Clouds"]
                ).execute_async()
                e = alix.aiolifx.TileEffectType[effect.upper()].value
                st = alix.TileEffectSkyType[sky_type.upper()].value
                sp = 50
            else:
                e = alix.aiolifx.TileEffectType[effect.upper()].value
                st = None
                sp = 3

            device.set_tile_effect(effect=e, speed=sp, sky_type=st)
            device = None
        elif feature == DeviceFeatures.MULTIZONE_FIRMWARE_EFFECT:

            print("Getting current firmware effect state from multizone device")
            device.get_multizone_effect(
                callb=lambda _, r: print(
                    f"\nCurrent effect={r.effect_str}"
                    f"\nSpeed={r.speed/1000 if getattr(r, 'speed', None) is not None else 0}"
                    f"\nDuration={r.duration/1000000000 if getattr(r, 'duration', None) is not None else 0:4f}"
                    f"\nDirection={r.direction_str}"
                )
            )
            device = None
        elif feature == DeviceFeatures.MULTIZONE_FIRMWARE_EFFECT_START_STOP:

            effect = await inquirer.fuzzy(
                message="Effect",
                choices=["Off", "Move"],
            ).execute_async()

            if effect.lower() == "off":
                device.set_multizone_effect(effect=0)
            else:
                direction = await inquirer.fuzzy(
                    message="Direction",
                    choices=["Left", "Right"],
                ).execute_async()

                e = alix.aiolifx.MultiZoneEffectType[effect.upper()].value
                d = alix.aiolifx.MultiZoneDirection[direction.upper()].value
                device.set_multizone_effect(effect=e, speed=3, direction=d)
            device = None
        elif feature == DeviceFeatures.HEV_CONFIGURATION:
            # Get current state
            print("Getting current HEV configuration")
            device.get_hev_configuration(
                callb=lambda _, r: print(
                    f"\nHEV: indication={r.indication}, " f"duration={r.duration}"
                )
            )

            set_hev_configuration = await inquirer.confirm(
                message="Set HEV configuration?", default=False
            ).execute_async()
            if set_hev_configuration:
                indication = await inquirer.confirm(
                    message="Indication?", default=False
                ).execute_async()
                duration = await inquirer.number(
                    "Duration (seconds)",
                    replace_mode=True,
                    min_allowed=0,
                    validate=EmptyInputValidator(),
                ).execute_async()
                print(
                    f"Configuring default HEV cycle with "
                    f"{'' if indication else 'no '}indication for "
                    f"{duration} second(s)"
                )
                device.set_hev_configuration(
                    indication=indication,
                    duration=duration,
                    callb=lambda _, r: print(
                        f"\nHEV: indication={r.indication}, " f"duration={r.duration}"
                    ),
                )
            device = None
        elif feature == DeviceFeatures.RELAYS:
            callback = lambda x, statePower: print(
                # +1 to use 1-indexing
                f"\nRelay {statePower.relay_index + 1}: {'On' if statePower.level == 65535 else 'Off'}"
            )
            device.get_rpower(callb=callback)
            await aio.sleep(
                0.5
            )  # to wait for the callback to be called. TODO: use await
            should_set_relay_state = await inquirer.confirm(
                message="Set relay state?", default=False
            ).execute_async()
            if should_set_relay_state:
                relay_index = await inquirer.number(
                    "Relay index",
                    replace_mode=True,
                    min_allowed=1,
                    max_allowed=4,
                    validate=EmptyInputValidator(),
                ).execute_async()
                relay_index = int(relay_index) - 1  # Convert to 0-indexing
                relay_state = await inquirer.select(
                    "Relay state", choices=["On", "Off"]
                ).execute_async()
                set_rpower = partial(device.set_rpower, relay_index, callb=callback)
                set_rpower(relay_state.lower() == "on")
                await aio.sleep(
                    0.5
                )  # to wait for the callback to be called. TODO: use await
            feature = None
        elif feature == DeviceFeatures.BUTTONS:

            def callback(x, buttonResponse):
                def get_action_name(action_index):
                    if action_index == 0:
                        return "Single Press"
                    elif action_index == 1:
                        return "Double Press"
                    elif action_index == 2:
                        return "Long Press"
                    else:
                        # To present 1-indexing to users
                        return f"Action {action_index + 1}"

                buttons_str = ""
                for button_index, button in enumerate(
                    buttonResponse.buttons[: buttonResponse.buttons_count]
                ):
                    buttons_str += f"\tButton {button_index + 1}:\n"
                    # At the moment, LIFX app only supports single, double and long press
                    MAX_ACTIONS = 3
                    for action_index, action in enumerate(
                        button["button_actions"][:MAX_ACTIONS]
                    ):
                        buttons_str += (
                            f"\t\t{get_action_name(action_index)}\n"
                            + f"\t\t\tGesture: {action['button_gesture']}\n"
                            + f"\t\t\t{action['button_target_type']}\n"
                            + f"\t\t\t{action['button_target']}\n"
                        )
                return print(
                    f"Count: {buttonResponse.count}\n"
                    + f"Index: {buttonResponse.index}\n"
                    + f"Buttons Count: {buttonResponse.buttons_count}\n"
                    + f"Buttons:\n{buttons_str}"
                )

            device.get_button(callback)
            await aio.sleep(0.5)
        elif feature == DeviceFeatures.BUTTON_CONFIG:

            def callback(x, buttonConfig):
                def get_backlight_str(backlight):
                    # Switch returns the kelvin value as a byte, so we need to convert it to a kelvin value
                    # The kelvin value is reversed (higher byte value = lower kelvin).
                    # Below 10495 and above 56574 are outside the range of supported Kelvin values
                    def get_kelvin(byte_value):
                        MIN_KELVIN_VALUE = 1500
                        MAX_KELVIN_VALUE = 9000
                        KELVIN_RANGE = MAX_KELVIN_VALUE - MIN_KELVIN_VALUE
                        MIN_BYTE_VALUE = 10495  # 9000 Kelvin
                        MAX_BYTE_VALUE = 56575  # 1500 Kelvin
                        BYTE_RANGE = MAX_BYTE_VALUE - MIN_BYTE_VALUE
                        if byte_value <= MIN_BYTE_VALUE:
                            return MAX_KELVIN_VALUE
                        elif byte_value < MAX_BYTE_VALUE:
                            return int(
                                round(
                                    MAX_KELVIN_VALUE
                                    - ((byte_value - MIN_BYTE_VALUE) / BYTE_RANGE)
                                    * KELVIN_RANGE
                                )
                            )
                        else:
                            return MIN_KELVIN_VALUE

                    backlight_color = {
                        "hue": int(round(360 * (backlight["hue"] / 65535))),
                        "saturation": int(
                            round(100 * (backlight["saturation"] / 65535))
                        ),
                        "brightness": int(
                            round(100 * (backlight["brightness"] / 65535))
                        ),
                        "kelvin": get_kelvin(backlight["kelvin"]),
                    }
                    return (
                        f"\n\tHue: {backlight_color['hue']},\n"
                        + f"\tSaturation: {backlight_color['saturation']}\n"
                        + f"\tBrightness: {backlight_color['brightness']}\n"
                        + f"\tKelvin (used if Hue is 0): {backlight_color['kelvin']}"
                    )

                backlight_on_color_str = get_backlight_str(
                    buttonConfig.backlight_on_color
                )
                backlight_off_color_str = get_backlight_str(
                    buttonConfig.backlight_off_color
                )

                return print(
                    f"Haptic Duration (ms): {buttonConfig.haptic_duration_ms}\nBacklight on color: {backlight_on_color_str}\nBacklight off color: {backlight_off_color_str}"
                )

            device.get_button_config(callback)
            await aio.sleep(0.5)
            should_set_button_config = await inquirer.confirm(
                message="Set button config?", default=False
            ).execute_async()
            if should_set_button_config:
                haptic_duration_ms = await inquirer.number(
                    "Haptic duration (ms)",
                    replace_mode=True,
                    default=30,
                    min_allowed=0,
                    max_allowed=1000,
                    validate=EmptyInputValidator(),
                ).execute_async()

                async def get_backlight_color():
                    color_set_mode = await inquirer.rawlist(
                        message="Set backlight via kelvin or color (hue, saturation)?",
                        choices=["Kelvin", "Color"],
                    ).execute_async()
                    # Switch accepts the actual kelvin value as the input
                    kelvin = 4500
                    hue = 0
                    saturation = 0
                    if color_set_mode == "Kelvin":
                        while True:
                            kelvin = await inquirer.number(
                                "Kelvin (1500 - 9000)",
                                min_allowed=0,
                                max_allowed=9000,
                                validate=EmptyInputValidator(),
                            ).execute_async()
                            if int(kelvin) < 1500 or int(kelvin) > 9000:
                                print("Kelvin must be greater within 1500 and 9000")
                                continue
                            break
                    else:
                        hue = await inquirer.number(
                            "Hue (0 - 360)",
                            min_allowed=0,
                            max_allowed=360,
                            validate=EmptyInputValidator(),
                        ).execute_async()
                        saturation = await inquirer.number(
                            "Saturation (0 - 100)",
                            min_allowed=0,
                            max_allowed=100,
                            validate=EmptyInputValidator(),
                        ).execute_async()
                    brightness = await inquirer.number(
                        "Brightness (0 - 100)",
                        min_allowed=0,
                        max_allowed=100,
                        validate=EmptyInputValidator(),
                    ).execute_async()
                    return {
                        "hue": int(round(65535 * (int(hue) / 360))),
                        "saturation": int(round(65535 * (int(saturation) / 100))),
                        "brightness": int(round(65535 * (int(brightness) / 100))),
                        "kelvin": int(kelvin),
                    }

                print("Backlight on color")
                backlight_on_color = await get_backlight_color()
                print("Backlight off color")
                backlight_off_color = await get_backlight_color()

                device.set_button_config(
                    haptic_duration_ms,
                    backlight_on_color,
                    backlight_off_color,
                    callback,
                )
                await aio.sleep(0.5)
        elif feature == DeviceFeatures.REBOOT:
            # Reboot bulb
            print(
                "Rebooting bulb in 3 seconds. If the bulb is on, it will flicker off and back on as it reboots."
            )
            print("Hit CTRL-C within 3 seconds to to quit without rebooting the bulb.")
            sleep(3)
            device.set_reboot()
            print("Bulb rebooted.")
            feature = None
    return


async def amain():
    global MyBulbs

    # Avoid any asyncio error message
    await aio.sleep(0)

    MyBulbs = bulbs()
    loop = aio.get_event_loop()
    discovery = alix.LifxDiscovery(loop, MyBulbs)
    try:
        discovery.start()
        print("Starting")
        # Wait to discover bulbs
        await aio.sleep(1)
        print("Use Ctrl-C to quit")
        print("Start typing or use arrow keys to navigate, and enter to select.")
        await readin()
    finally:
        discovery.cleanup()


@click.command(help="Track and interact with Lifx light bulbs.")
@click.option(
    "-6",
    "--ipv6prefix",
    default=None,
    help="Connect to Lifx using IPv6 with given /64 prefix (Do not end with colon unless you have less than 64bits).",
)
@click.option(
    "-x",
    "--extra",
    is_flag=True,
    default=False,
    help="Print unexpected messages.",
)
def cli(ipv6prefix, extra):
    global opts
    opts = {"ipv6prefix": ipv6prefix, "extra": extra}
    try:
        aio.run(amain())
    except KeyboardInterrupt:
        print("\nExiting at user's request.")
    except Exception as e:
        print(f"Error: {e}")


if __name__ == "__main__":
    cli()



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
import setuptools

version = "1.0.8"

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aiolifx",
    packages=["aiolifx"],
    version=version,
    author="François Wautier",
    author_email="francois@wautier.eu",
    description="API for local communication with LIFX devices over a LAN with asyncio.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/aiolifx/aiolifx",
    keywords=["lifx", "light", "automation"],
    license="MIT",
    install_requires=[
        "async_timeout>=3.0.1",
        "bitstring",
        "ifaddr",
        "click>=8.1.0,<8.2.0",
        "InquirerPy>=0.3.0,<0.4.0",
    ],
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # Pick your license as you wish (should match "license" above)
        "License :: OSI Approved :: MIT License",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3",
    ],
    entry_points={"console_scripts": ["aiolifx = aiolifx.__main__:cli"]},
    python_requires=">=3.4",
    zip_safe=False,
)
